declare namespace CloudbaseIndexChecker {
  export const getIndexSuggestion: (options: IGetIndexSuggestionOptions) => IIndexSuggestion
  export const getQueryKeyPattern: (spec: IQuerySpec) => Record<string, any>

  interface IGetIndexSuggestionOptions {
    spec: IQuerySpec
    indices: IIndexInfo[]
  }
  
  interface IIndexSuggestion {
    // normal: any[]
    perfect: IPerfectCandidate[]
    warnings: IIndexWarning[]
  }
  
  interface IPerfectCandidate {
    keys: IKeyIndexInfo[]
    samePrefixIndices: IIndexInfo[]
  }
  
  interface IIndexWarningFullScan {
    type: 'FULL_SCAN'
    reason: 'EMPTY_QUERY' | 'AGGREGATE_FIRST_STAGE_NOT_MATCH'
  }
  
  interface IIndexWarningRegExp {
    type: 'REGEXP'
    field: string
    reason: 'CASE_INSENSITIVE' | 'NOT_PREFIX_EXPR'
  }
  
  interface IIndexWarningUnselectiveOperator {
    type: 'UNSELECTIVE_OPERATOR'
    field: string
    operator: string
  }
  
  type IIndexWarning = IIndexWarningFullScan | IIndexWarningRegExp | IIndexWarningUnselectiveOperator
  
  interface IQuerySpec {
    query: Record<string, any>
    sort?: {
      field: string
      type: 1 | -1
    }[]
    projection?: Record<string, number>
  }
  
  interface IOrder {
    field: string
    direction: ORDER_DIRECTION
  }
  
  enum ORDER_DIRECTION {
    ASC = 0,
    DESC = 1,
  }
  
  interface IIndexInfo {
    name: string
    keys: IKeyIndexInfo[]
  }
  
  interface IKeyIndexInfo {
    field: string
    type: 1 | -1 | '2dsphere'
  }
}

export = CloudbaseIndexChecker
